// userinterface.c  // ********************************************************
//
// user interface related routines for the project "USB-MidiCom",
// simple data converter for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Author and (c)2016-21 Wolfgang Schemmert. The copyright claim and
// the GPL-3 Licence is restricted to this file and the depending files
// "main.c", "main.h, ""usb-fs.c" and "globaldefines.h" .
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: www.midi-and-more.de
// A detailled hardware construction 
// and operation manual is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************


#include "stm32l4xx.h" //includes "stm32l476xx.h"
#include "globaldefines.h"

extern struct {            //half words for code compatibility with STM32F042,
uint16_t ComBaud;	//which allows only half word flash write
uint16_t MidiBaud;
uint16_t Channel;
uint16_t Hysteresis;
uint16_t VidH;
uint16_t VidL;
uint16_t PidH;
uint16_t PidL;
//user count of inputs starts from '1'
//the offset for a MIDI msg entry relative to "PidH" is = (user I/O num *2)
//new configuration REVNUM 8++: bits 0-7 of MsgXD =3 byte msg 1st data byte 
//special case=highByte=0x80 or 0x81: send analog input, else digital level
//else last data byte to be sent in digital message
uint16_t Msg1S;
uint16_t Msg1D;
uint16_t Msg2S;
uint16_t Msg2D;
uint16_t Msg3S;
uint16_t Msg3D;
uint16_t Msg4S;
uint16_t Msg4D;
uint16_t Msg5S;
uint16_t Msg5D;
uint16_t Msg6S;
uint16_t Msg6D;
uint16_t Msg7S;
uint16_t Msg7D;
uint16_t Msg8S;
uint16_t Msg8D;
uint16_t Msg9S;
uint16_t Msg9D;
uint16_t Msg10S;
uint16_t Msg10D;
uint16_t Msg11S;
uint16_t Msg11D;
uint16_t Msg12S;
uint16_t Msg12D;
uint16_t Msg13S;
uint16_t Msg13D;
uint16_t Msg14S;
uint16_t Msg14D;
uint16_t Msg15S;
uint16_t Msg15D;
uint16_t Msg16S;
uint16_t Msg16D;
uint16_t MsgUS;
uint16_t MsgUD;
uint16_t NumBase;	
uint16_t dummy;	//dummy necessary: flash is programmed in double words
} gFlashMirror, gRuntimeSetup;//28 halfwords= 56bytes= 14 words= 7 double words
//gFlashMirror contains modified setup to be written to Flash
//gRuntimeSetup contains active working setup, possibly changed during session

extern uint16_t gInState[IO_NUM +1];		//index0 is never used

extern uint32_t gStartSetup, gUsbStartSetup, gSetupMode;
extern uint32_t gExecOn, gExecBlock, gFlashChanged, gNumBase;

extern const uint8_t gTypeString [];
extern const uint8_t gCopyString [];
extern void configure_GPIO(void);
extern void configurePort(uint32_t);
extern void msDelay(uint16_t);
extern void initADC(void);
//extern void disableADC(void);
extern void sendAllB(uint8_t);
extern void sendAllNibble(uint8_t);
extern void sendAllX8(uint8_t);
extern void sendAllS12(uint16_t);
extern void sendAllX16(uint16_t);
extern void sendX32(uint32_t);			//for debug only
extern void sendAllString(const uint8_t *);
extern void sendCRLF(void);
extern uint32_t pullAllRxBuf(void);
extern void popFlashPage15(void);
extern void pushFlashPage15(void);

// ****************************************

void userConfig(void);
void setOnOff(void);
void setVidPid(void);
void setBaudrate(void);
void setMidiBaud(void);
void setMidiChannel(void);
void setHysteresis(void);
void configMsg(uint8_t);
uint16_t parseStatus(void);
uint8_t parseData(void);
uint8_t parseMsgType(void);
uint8_t parseHexNibble(uint8_t);
uint8_t parseDeciDigit(void);
void listCommands(void);
void sendConfiguration(void);
void sendBaudRate(const uint8_t);


const uint8_t gcConfigStr[] = {"Configuration:"};
const uint8_t gcVidPidStr[] = {"USB Vid/Pid of this device (hex)"};
const uint8_t gcComBaudStr[] ={"Serial COM Baudrate (active in RS-232 Mode)"};
const uint8_t gcMidiBaudStr[] = {"MIDI Baudrate (active in MIDI Mode)"};
const uint8_t gcChannelStr[] = {"Global MIDI Channel(1-16)"};
const uint8_t gcHysteresisStr[] = {"Hysteresis(0..9)"};
const uint8_t gcNumBStr[] = {"NumberBase="};
const uint8_t gcDecimalStr[] = {"decimal"};
const uint8_t gcHexStr[] = {"hex"};

const uint8_t gcInputStr[17] [14] = {{"Input1 (PA2):\0"},{"Input2 (PA3):\0"},
                                     {"Input3 (PB0):\0"},{"Input4 (PA4):\0"},
                                     {"Input5 (PA1):\0"},{"Input6 (PA0):\0"},
									 {"Input7(PA15):\0"},{"Input8 (PB7):\0"},
									 {"Input9 (PB3):\0"},{"Input10(PB5):\0"},
                                     {"Input11(PB4):\0"},{"Input12(PA7):\0"},
									 {"Input13(PA6):\0"},{"Input14(PA5):\0"},
                                     {"Input15(PB9):\0"},{"Input16(PB8):\0"},
									 {"InputU(PC13):\0"}};
const uint8_t gcNoMsgStr[] = {" NO msg configured"};


void userConfig(void)  // ***************************************************
// **************************************************************************
{
uint16_t recvword;
uint8_t msgbyte,k;
const uint8_t saveQuestString[] = {"Setup has changed. Save with '|' for permanent storage in Flash"};
const uint8_t cancelString[] = {"       Cancel Exit with <ESC>"};
const uint8_t SaveAgainStr[] = {"Save to Flash? ->type again "};
const uint8_t NotSavedStr[] = {" NOT SAVED! "};
const uint8_t LoadAgainStr[] = {" Reload Setup from Flash? ->type again "};
const uint8_t NotLoadedStr[] = {" NOT RELOADED! "};
const uint8_t IoNumStr[] = {"Config Input# "};
//opening message:
    sendCRLF();
	sendAllString(gTypeString);
    sendAllB(HARDWARE_VERSION+0x30);
    sendAllB('.');
    sendAllB(REVNUM+0x30);
	sendAllString(gCopyString);
	msDelay(1000);
    msgbyte = 0;

//permanent loop until Exit for command handling
    while (1)   {
        sendCRLF();
        sendAllB('C');
        sendAllB('M');
        sendAllB('D');
        sendAllB('>');
NEXTBYTE:
        while ((recvword = pullAllRxBuf()) == 0) ;
        msgbyte = (uint8_t)(recvword & 0x00FF);
        if ((msgbyte == 0x20) || (msgbyte == 0x0A)) goto NEXTBYTE; //ignore
        if ((msgbyte > 0x60) && (msgbyte < 0x7B)) msgbyte -= 0x20;
                                                //force uppercase letter
		if ((msgbyte >= '1') && (msgbyte <= '9'))  {
			sendAllString(IoNumStr);
			sendAllB(msgbyte);
			if (msgbyte != '1')  sendAllB(':');
		}
		else  {
			sendAllB(msgbyte);
			sendAllB(':');
			sendAllB(0x20);
		}
        switch (msgbyte)  {
            case '!':
                sendConfiguration();
                break;

            case '?':
                listCommands();
                break;

            case 'V':
                setVidPid();
                break;

            case 'B':
                setBaudrate();
                break;
            case 'M':
                setMidiBaud();
                break;

            case 'C':
                setMidiChannel();
                break;

            case 'H':
                setHysteresis();
                break;

			case 'D':
                gNumBase = 0;
				sendAllString(gcNumBStr);
				sendAllString(gcDecimalStr);
                break;

            case 'X':
                gNumBase = 1;		//default
				sendAllString(gcNumBStr);
				sendAllString(gcHexStr);
                break;

            case '1':
				if((k = parseDeciDigit()) == '?')  {
					sendAllB('?');
					break;
				}
				if (k == 0x0D)  {
#ifdef STLINK_CUTOFF
					sendAllB(':');
					configMsg(1);
					break;
#else
					sendAllB('?');
					break;
#endif
				}
				else  {
					if (k > 6)  {
					sendAllB('?');
					break;
					}
					sendAllB(':');
					k += 10;
					configMsg(k);
					break;
				}
				break;
#ifdef STLINK_CUTOFF
            case '2':
                configMsg(2);
                break;
#endif
            case '3':
                configMsg(3);
                break;
            case '4':
                configMsg(4);
                break;
            case '5':
                configMsg(5);
                break;
            case '6':
                configMsg(6);
                break;
            case '7':
                configMsg(7);
                break;
            case '8':
                configMsg(8);
                break;
            case '9':
                configMsg(9);
                break;
            case 'U':				//User button
                configMsg(17);
                break;

            case '@':
                sendAllB(0x0D);
                sendAllB('>');
                msDelay(300);
                if (pullAllRxBuf() != 0) {
                    sendAllB('?');            //pause <300ms too short
                    sendAllB(0x20);
                }
                sendAllString(LoadAgainStr);
                do  {
                    recvword = pullAllRxBuf();
                }  while (recvword == 0) ;
                if (recvword == 0x8040)  {      //'@'
                    popFlashPage15();
                    configure_GPIO();
                    sendAllB(0x20);
                    sendAllB('o');
                    sendAllB('k');
                    sendAllB(0x20);
                    gFlashChanged = 0;
                }
                else sendAllString(NotLoadedStr);
                break;

            case '~':               //Save Setup to Flash if changed
                sendAllB(0x0D);
                sendAllB('>');
                msDelay(300);
                if (pullAllRxBuf() != 0) {
                    sendAllB('?');            //pause <300ms too short
                    sendAllB(0x20);
                }
                sendAllString(SaveAgainStr);
                do  {
                    recvword = pullAllRxBuf();
                }  while (recvword == 0) ;
                if (recvword == 0x807C)  {      //'|'
                    pushFlashPage15();
                    sendAllB(0x20);
                    sendAllB('o');
                    sendAllB('k');
                    sendAllB(0x20);
                    gFlashChanged = 0;
                }
                else sendAllString(NotSavedStr);
                break;

            case 'E':
                if ((gFlashChanged & 0x00FFFFFF) != 0)  {
					gExecBlock = 1;
                    sendAllString(saveQuestString);
                    sendCRLF();
                    sendAllString(cancelString);
                    sendCRLF();
                    sendAllB('>');
                    while ((recvword = pullAllRxBuf()) == 0) ;
					if (recvword == 0x801B)  {		//ESC
						gExecBlock = 0;
						break;
					}
                    if (recvword == 0x807C)  {		//'|'
                        msDelay(300);
                        if (pullAllRxBuf() != 0) {
                            sendAllB('?');			//pause <300ms too short
                            sendAllB(0x20);
                        }
                        sendAllString(SaveAgainStr);
                        do  {
                            recvword = pullAllRxBuf();
                        }  while (recvword == 0) ;
                        if (recvword == 0x807C)  {	//'|'
                            pushFlashPage15();
                            sendAllB(0x20);
                            sendAllB('o');
                            sendAllB('k');
                            sendAllB(0x20);
                        }
                        else sendAllString(NotSavedStr);
                    }
                    else  sendAllString(NotSavedStr);
                }
                sendAllB('E');
                sendAllB('x');
                sendAllB('i');
                sendAllB('t');
                sendCRLF();
                gSetupMode = 0;
                gStartSetup = 0;
                gUsbStartSetup = 0;
				gExecBlock = 0;
                return;
            case 0x0D:
                sendAllB(0x0D);
                break;
            default:
                sendAllB('?');
                break;
        }
    }
}


void setVidPid(void)  // ****************************************************
// **************************************************************************
{
uint8_t nibble;
uint16_t VidH, VidL, PidH, PidL;

const uint8_t vidStr[] = {" enter new Vid (4 hex nibbles): "};
const uint8_t pidStr[] = {" enter new Pid (4 hex nibbles): "};

	gExecBlock = 1;
	gExecOn = 0;		//no input change messages during configuration
	sendCRLF();
	sendAllString(vidStr);
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    VidH = (uint16_t)nibble;
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    VidH = (VidH<<4) + (uint16_t)nibble;                // *16
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    VidL = (uint16_t)nibble;
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    VidL = (VidL<<4) + (uint16_t)nibble;

    if ((VidH | VidL) == 0x0000)  {
        VidH = 0xFFFF;      //re-virginate flash entry
        VidL = 0xFFFF;
    }
    sendCRLF();
    sendAllString(pidStr);
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    PidH = (uint16_t)nibble;
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    PidH = (PidH<<4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    PidL = (uint16_t)nibble;
    if ((nibble = parseHexNibble(0)) == 0xEE)  {
		gExecBlock = 0;
		return;
	}
    PidL = (PidL<<4) + (uint16_t)nibble;
    if ((PidH | PidL) == 0x0000)  {
        PidH = 0xFFFF;      //re-virginate flash entry
        PidL = 0xFFFF;
    }
    gFlashMirror.VidH =  VidH;  //not for runtime operation, only save to Flash
    gFlashMirror.VidL =  VidL;
    gFlashMirror.PidH =  PidH;
    gFlashMirror.PidL =  PidL;
    gFlashChanged |= 0x0000FF00;
	gExecBlock = 0;
}


void setBaudrate(void)  // **************************************************
// **************************************************************************
{
uint8_t digit, baudrate;
const uint8_t baudStr[] = {"COM baud rate - enter first 2 digits: "};

	gExecBlock = 1;
    sendCRLF();
	sendAllString(baudStr);
    if((digit = parseDeciDigit()) == '?')  {
		gExecBlock = 0;
		return;
	}
    baudrate = digit;
    if ((digit== 0)||(digit== 4)||(digit== 6)||(digit== 7)||(digit== 8)) {
        sendAllB('?');
		gExecBlock = 0;
        return;
    }
    if((digit = parseDeciDigit()) == '?')  {
		gExecBlock = 0;
		return;
	}
    baudrate = (baudrate * 10) + digit;
    switch(baudrate)  {
    case 96:
        sendAllB('0');
        sendAllB('0');
        break;
    case 19:
        sendAllB('2');
        sendAllB('0');
        sendAllB('0');
        break;
     case 31:
        sendAllB('2');
        sendAllB('5');
        sendAllB('0');
        break;
    case 38:
        sendAllB('4');
        sendAllB('0');
        sendAllB('0');
        break;
    case 57:
        sendAllB('2');
        sendAllB('0');
        sendAllB('0');
        break;
    case 11:
        sendAllB('5');
        sendAllB('2');
        sendAllB('0');
        sendAllB('0');
        break;
      case 23:
        sendAllB('0');
        sendAllB('4');
        sendAllB('0');
        sendAllB('0');
        break;
    default:
        sendAllB('?');
		gExecBlock = 0;
        return;
    }
    gFlashMirror.ComBaud = (uint16_t)baudrate; //only save to Flash
    gFlashChanged = 0x000000FF;
	gExecBlock = 0;
}


void setMidiBaud(void)  // **************************************************
// **************************************************************************
{
uint8_t digit, baudrate;
const uint8_t baudStr[] = {"MIDI baud rate - enter first 2 digits: "};

	gExecBlock = 1;
    sendCRLF();
	sendAllString(baudStr);
    if((digit = parseDeciDigit()) == '?')  {
		gExecBlock = 0;
		return;
	}
    baudrate = digit;
    if ((digit== 0)||(digit== 2)||(digit== 4)||(digit== 6)||(digit== 7)||
																(digit== 8)) {
        sendAllB('?');
		gExecBlock = 0;
        return;
    }
    if((digit = parseDeciDigit()) == '?')  {
		gExecBlock = 0;
		return;
	}
    baudrate = (baudrate * 10) + digit;
    switch(baudrate)  {
    case 96:
        sendAllB('0');
        sendAllB('0');
        break;
    case 19:
        sendAllB('2');
        sendAllB('0');
        sendAllB('0');
        break;
    case 31:
        sendAllB('2');
        sendAllB('5');
        sendAllB('0');
        break;
    case 38:
        sendAllB('4');
        sendAllB('0');
        sendAllB('0');
        break;
    case 57:
        sendAllB('2');
        sendAllB('0');
        sendAllB('0');
        break;
    case 11:
        sendAllB('5');
        sendAllB('2');
        sendAllB('0');
        sendAllB('0');
        break;
    default:
        sendAllB('?');
		gExecBlock = 0;
        return;
    }
    gFlashMirror.MidiBaud = (uint16_t)baudrate; //only save to Flash
    gRuntimeSetup.MidiBaud = (uint16_t)baudrate; //for actual operation
    gFlashChanged |= 0x00FF0000;
	gExecBlock = 0;
}


void setMidiChannel(void)  // ***********************************************
//get the common MIDI channel in decimal format 1...16
// **************************************************************************
{
uint8_t digit, channel;
const uint8_t channelStr[] = {"Enter MIDI channel (1...16): "};

	gExecBlock = 1;
    sendCRLF();
	sendAllString(channelStr);
    if((digit = parseDeciDigit()) == '?')  {
		gExecBlock = 0;
		return;
	}
    if ((digit == 0x0A) || (digit == 0x0D)) {
        sendAllB('?');
		gExecBlock = 0;
        return;
    }
    channel = digit;
    if((digit = parseDeciDigit()) == '?')  {
		gExecBlock = 0;
		return;
	}
    if (digit != 0x0D) channel = (channel<<3) + (channel<<1) + digit; // *10
    channel --;         //conversion to "MIDI internal" channel number
    if (channel > 15) { //includes user entry "0" due to unsigned underflow
        sendAllB(' ');
        sendAllB('1');
        sendAllB('-');
        sendAllB('1');
        sendAllB('6');
        sendAllB('!');
        sendAllB('?');
        return;
    }
    gFlashMirror.Channel = (uint16_t)channel;//to be saved to Flash
    gRuntimeSetup.Channel = (uint16_t)channel;//immediately,actual operation
    gFlashChanged |= 0x00FF0000;
	gExecBlock = 0;
}

void setHysteresis(void)  // ***********************************************
//Default = 4. Max. recommended value = 6. Value 0 results in lot of garbage
// **************************************************************************
{
uint8_t digit;

	gExecBlock = 1;
    sendCRLF();
	sendAllString(gcHysteresisStr);
	sendAllB(':');
	sendAllB(' ');
    if((digit = parseDeciDigit()) == '?')  {
		gExecBlock = 0;
		return;
	}
    if ((digit == 0x0A) || (digit == 0x0D)) {
        sendAllB('?');
		gExecBlock = 0;
        return;
    }
    gFlashMirror.Hysteresis = (uint16_t)digit;	//to be saved to Flash
    gRuntimeSetup.Hysteresis = (uint16_t)digit;	//immediately,actual operation
    gFlashChanged |= 0x00FF0000;
	gExecBlock = 0;
}


void configMsg(uint8_t number)  // ******************************************
// **************************************************************************
{
uint8_t type, statusType, fullStatus;
uint16_t status;
uint16_t data;
uint16_t *fMirrorPtr, *runtimePtr;

const uint8_t statusStr[] = {"Status(hex)? "};
const uint8_t numberStr[] = {" ->1st MIDI data byte("};
const uint8_t dataTypeStr[] = {"  Port Read:'D'=digital,'A'=analog? "};
const uint8_t pitchTypeStr[] = {"  Port Read:'D'=digital,'A'=analog(7bit),'B'=analog(12bit)? "};
const uint8_t digOnlyStr[] =  {"  Port Read: digital only"};
const uint8_t digMsgStr[] = {"  last MIDI data byte when button pushed("};

	gExecBlock = 1;			//no input change messages during configuration
//in struct gFlashMirror the offset relative to PidH is = (user I/Onumber *2)
    fMirrorPtr = &(gFlashMirror.PidH) + (number<<1); //to be saved to Flash
    runtimePtr = &(gRuntimeSetup.PidH) + (number<<1);  //actual operation
    data = 0;
	
	sendCRLF();
    sendAllString(statusStr);
    status = parseStatus();

    if (status > 0x00FF) {			//parse error: nonsense input
        sendAllB('?');
		gExecBlock = 0;
        return;
    }
    if (status == 0)  {				//S00-->delete Msg in flash
        *fMirrorPtr = 0xFFFF;
        *(fMirrorPtr+1) = 0xFFFF;
        *runtimePtr = 0xFFFF;
        *(runtimePtr+1) = 0xFFFF;
		configurePort(number);
        gFlashChanged |= 0x00FF0000;
		gExecBlock = 0;
        return;
    }
	if (status >= 0x00F6)  {
		*fMirrorPtr++ = status;
		*fMirrorPtr = 0x0080;		//to be distinguised from empty entry
		*runtimePtr++ = status;
		*runtimePtr = *fMirrorPtr;
		configurePort(number);
		gFlashChanged |= 0x00FF0000;
		gExecBlock = 0;
		return;
	}
	if (status <= 0x000F) statusType = (uint8_t)status;
    else statusType = ((uint8_t)status >> 4) & 0x0F;
	if (statusType < 9)  {
        sendAllB('?');
		gExecBlock = 0;
        return;
	}
    if ((statusType <= 0x0D) || (status == 0x00F2) || (status == 0x00F3)) {
		if ((statusType != 0x0C)&&(statusType != 0x0D)&&(status != 0x00F3)) {
			sendAllString(numberStr);	//parse data1 of 3 byte MIDI message
			if (gNumBase) sendAllString(gcHexStr);
			else  sendAllString(gcDecimalStr);
			sendAllB('?');
			sendAllB(0x29);		//')'
			sendAllB(' ');
			data = (uint16_t)parseData();
			if (data > 0x007F)  {			//parse error: nonsense input
				gExecBlock = 0;
				return;
			}
		}
		sendCRLF();
		if (((number>=7) && (number<=11)) || (number >= 15))  { //dig msg only
			sendAllString(digOnlyStr);
			type = 0;
		}
		else  {
			sendAllString(dataTypeStr);
			type = parseMsgType();
			if ((type != 0x0A) && (type != 0x0D)) {
				sendAllB('?');
				gExecBlock = 0;
				return;
			}
			if (type == 0x0A)  type = 0x80;
			else type = 0;
		}
    }
	else if (statusType == 0x0E)  {		//12 or 7 bit resolution posssible
		data = 0;						//data1 is always 7 LSB of 14bit msg
		if (((number>=7) && (number<=11)) || (number >= 15))  { //dig msg only
			sendAllString(digOnlyStr);
			type = 0;
		}
		else  {
			sendCRLF();
			sendAllString(pitchTypeStr);
			type = parseMsgType();
			if ((type != 0x0A) && (type != 0x0B) && (type != 0x0D)) {
				sendAllB('?');
				gExecBlock = 0;
				return;
			}
			if (type == 0x0A)  type = 0x80;
			else if (type == 0x0B)  type = 0x81;
			else type = 0;
		}
    }
	else {
        sendAllB('?');
		gExecBlock = 0;
        return;
	}
	if (type == 0)  {
		sendCRLF();
		sendAllString(digMsgStr);
		if (gNumBase) sendAllString(gcHexStr);
		else  sendAllString(gcDecimalStr);
		sendAllB('?');
		sendAllB(0x29);		//')'
		sendAllB(' ');
		type = parseData();
		if ((type > 0x7F) || (type == 0))  {	//parse error: nonsense input
			sendAllB('?');						//0 reserved for release msg)
			gExecBlock = 0;
			return;
		}
	}
	data = data | ((uint16_t)type << 8);

    *fMirrorPtr++ = status;
    *fMirrorPtr = data;
    *runtimePtr++ = status;
    *runtimePtr = *fMirrorPtr;
    configurePort(number);
	if ((ADC1->CR & ADC_CR_ADEN) == 0) initADC();
    gFlashChanged |= 0x00FF0000;
	gExecBlock = 0;
}


uint16_t parseStatus(void)   //**********************************************
// input is interpreted HEXADECIMAL (entered without leading 0x...!)
// small values (0x08 to 0x0E) will insert gGlobalMidiChannel,
// big values (0x80 to 0xF2) will insert individually entered MIDI channel
// at runtime
// **************************************************************************
{
uint8_t nibble;
uint16_t status;

    status = 0;

    if ((nibble= parseHexNibble(1)) >= 0xEE) return 0xFF00;
	status = (uint16_t)nibble << 4;					//High nibble
    if ((nibble = parseHexNibble(1)) == 0xEE) return 0xFF00;
    if (nibble == 0xFF) {				  //single nibble terminated with CRLF
		if (status == 0) return status;				//S00-->delete flash entry
		status = status >> 4;
		if ((status < 0x0009) || (status > 0x000E)) return 0xFF00;
		return status;								//HiNibble=0:"small value"
	}
	else status |= (uint16_t)nibble;				//Low nibble=MIDI Channel
    if (status == 0) return status;					//S00-->delete flash entry
    if (status < 0x0090) return 0xFF00;				//data range not valid
    if ((status == 0x00F0) || (status == 0x00F7)) return 0xFF00;	//SysEx
    if ((status == 0x00F1) || (status == 0x00F4) || (status == 0x00F5) ||
						(status==0x00F9)||(status==0x00FD)) return 0xFF00;
											//not defined, 0xF1 notsupported
    return status;
}


uint8_t parseMsgType(void)   //**********************************************
// input is interpreted HEXADECIMAL  0x00 to 0x7F !
// exception:if 0x80 is entered, measured analog input will be inserted in msg
// only applicable for selected inputs which support analog conversion.
// for 3 byte MIDI messages only accepted for 2nd data byte
// **************************************************************************
{
uint8_t nibble;

    if ((nibble= parseHexNibble(1)) == 0xEE) return 0xFF;
    return nibble;
}


uint8_t parseData(void)   //************************************************
// input is interpreted HEXADECIMAL  0x00 to 0x7F !
// exception:if 0x80 is entered, measured analog input will be inserted in msg
// only applicable for selected inputs which support analog conversion.
// for 3 byte MIDI messages only accepted for 2nd data byte
// **************************************************************************
{
uint8_t data, nibble;

    data = 0;
	
	if (gNumBase)  {			//enter hex format
		if ((nibble= parseHexNibble(1)) == 0xEE) return 0xFF;
		data = nibble;
		if ((nibble = parseHexNibble(1)) == 0xEE) return 0xFF;
		if (nibble == 0xFF) return data;		//input terminated by RETURN
		data = (data<<4) | nibble;				// *16
		return data;
	}
	else  {
		if ((nibble = parseDeciDigit()) == '?') return 0xFF;
		data = nibble;
		if ((nibble = parseDeciDigit()) == '?') return 0xFF;
		if (nibble == 0x0D) return data;
		data = (data * 10) + nibble;
		if ((nibble = parseDeciDigit()) == '?') return 0xFF;
		if (nibble == 0x0D) return data;
		data = (data * 10) + nibble;
		return data;
	}
}


uint8_t parseHexNibble(uint8_t mode)   //************************************
// **************************************************************************
{
 uint16_t recvword;
 uint8_t nibble;

    while ((recvword = pullAllRxBuf()) == 0) ;
    nibble = (uint8_t)(recvword & 0x00FF);
    if (mode && ((nibble == 0x0A)||(nibble == 0x0D)))  return 0xFF;
	if (nibble == 0x3F)  {				//'?' = 0x3F, must be selected here
        sendAllB('?');
        return 0xEE;
	}
    if ((nibble > 0x60) && (nibble < 0x7B))  nibble -= 0x20; //force uppercase
    if ((nibble >= 'A') && (nibble < 'G')) nibble -= 7;

    if  ((nibble >= 0x30) && (nibble < 0x40))  {
		nibble -= 0x30;
		sendAllNibble(nibble);
		return (nibble);
	}
    else {
        sendAllB('?');
        return 0xEE;
    }

}


uint8_t parseDeciDigit(void)   //********************************************
// **************************************************************************
{
uint16_t recvword;
 uint8_t digit;

    while ((recvword = pullAllRxBuf()) == 0) ;
    digit = (uint8_t)(recvword & 0x00FF);
    if ((digit == 0x0A) || (digit==0x0D)) return 0x0D;
    if ((digit < 0x30) || (digit > 0x39 )) {
        sendAllB('?');
        return '?';
    }
    sendAllB(digit);
    digit -= 0x30;     //ASCII>num
    return digit;
}


void listCommands(void) // **************************************************
// **************************************************************************
{
const uint8_t SetupStr[] = {"List of Setup Commands:"};
const uint8_t ActualStr[] = {"list Actual Setup"};
const uint8_t SetStr[] = {"set "};
const uint8_t IONumStr[] = {"1,2,3,4,5,6,7,8,9,10,11,12,13,14,1,1,16,U:"};
const uint8_t IOStr[] =	 {"   set MIDI msgs released by change of I/O no 1..9 +UserButton"};
const uint8_t LoadFlashStr[] = {"@: reload setup from Flash (changes since last PowerUp are cancelled)"};
const uint8_t SaveFlashStr[] = {"|: save new setup permanently in Flash (will become active at next Power Up)"};
const uint8_t ExitStr[] = {"E: exit SETUP, return to bidirectional communication"};

    sendCRLF();
    sendAllString(SetupStr);
    sendCRLF();

    sendAllB('!');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(ActualStr);
    sendCRLF();

    sendAllB('V');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(SetStr);
    sendAllString(gcVidPidStr);
    sendCRLF();

    sendAllB('B');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(SetStr);
    sendAllString(gcComBaudStr);
    sendCRLF();

    sendAllB('M');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(SetStr);
    sendAllString(gcMidiBaudStr);
    sendCRLF();

    sendAllB('C');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(SetStr);
    sendAllString(gcChannelStr);
    sendCRLF();

    sendAllB('H');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(SetStr);
    sendAllString(gcHysteresisStr);
    sendCRLF();

	sendAllB('D');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(SetStr);
    sendAllString(gcNumBStr);
    sendAllString(gcDecimalStr);
    sendCRLF();

    sendAllB('X');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(SetStr);
    sendAllString(gcNumBStr);
    sendAllString(gcHexStr);
    sendCRLF();

    sendAllString(IONumStr);
	sendCRLF();
    sendAllString(IOStr);
    sendCRLF();

    sendAllString(LoadFlashStr);
    sendCRLF();

    sendAllString(SaveFlashStr);
    sendCRLF();

    sendAllString(ExitStr);
}


void sendConfiguration(void) // *********************************************
// **************************************************************************
{
uint16_t msgword;
uint8_t msgbyte;
uint8_t k, n;
uint16_t *msgPtr_S, *msgPtr_D;
uint8_t statusType, fullStatus;

const uint8_t notYetStr[] = {"      (saved, not yet active, new:"};
const uint8_t notSavedStr[] = {"      (modified, not saved, new:"};
const uint8_t inputMsgStr[] = {"Input Change messages:"};
const uint8_t msgStr[] = {" configured msg "};
const uint8_t actualStr[] = {"Actual "};
const uint8_t digStr[] =   {" Dig:send data if pushed"};
const uint8_t dig2Str[] =   {" Dig:send data2 if pushed"};
const uint8_t ana7Str[] =  {" Ana7bit"};
const uint8_t ana12Str[] = {" Ana12bit"};
const uint8_t singleStr[] = {"Single byte msg"};

    sendCRLF();
    sendAllString(gcConfigStr);
    sendCRLF();

    sendAllB('V');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(actualStr);
    sendAllString(gcVidPidStr);
	sendAllB(':');
	sendAllB(' ');
    if ((gRuntimeSetup.VidH & gRuntimeSetup.VidL) == 0xFFFF)  \
                                        msgword = (DEFAULT_VIDH << 8) | DEFAULT_VIDL;
    else msgword =(gRuntimeSetup.VidH << 8)|(gRuntimeSetup.VidL & 0x00FF);
    sendAllX16(msgword);
    sendAllB('/');
    sendAllB(' ');
    if ((gRuntimeSetup.PidH & gRuntimeSetup.PidL) == 0xFFFF)  \
                                        msgword = (DEFAULT_PIDH << 8) | DEFAULT_PIDL;
    else msgword =(gRuntimeSetup.PidH << 8)|(gRuntimeSetup.PidL & 0x00FF);
    sendAllX16(msgword);
    if ((gFlashChanged & 0xFF00FF00) == 0x0000FF00) {
		sendCRLF();
		sendAllString(notSavedStr);
		msgword =(gFlashMirror.VidH << 8)|(gFlashMirror.VidL & 0x00FF);
		sendAllX16(msgword);
		msgword =(gFlashMirror.PidH << 8)|(gFlashMirror.PidL & 0x00FF);
		sendAllX16(msgword);
	}
    else if((gFlashChanged & 0xFF00FF00) == 0xFF00FF00) {
		sendCRLF();
		sendAllString(notYetStr);
		msgword =(gFlashMirror.VidH << 8)|(gFlashMirror.VidL & 0x00FF);
		sendAllX16(msgword);
		msgword =(gFlashMirror.PidH << 8)|(gFlashMirror.PidL & 0x00FF);
		sendAllX16(msgword);
	}
    sendCRLF();

    sendAllB('B');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(actualStr);
    sendAllString(gcComBaudStr);
	sendAllB(':');
	sendAllB(' ');
	msgbyte = (uint8_t)gRuntimeSetup.ComBaud;
	sendBaudRate(msgbyte);
    if ((gFlashChanged & 0xFF0000FF) == 0x000000FF) {
		sendCRLF();
		sendAllString(notSavedStr);
		sendBaudRate((uint8_t)gFlashMirror.ComBaud);
	}
    else if((gFlashChanged & 0xFF0000FF) == 0xFF0000FF) {
		sendCRLF();
		sendAllString(notYetStr);
		sendBaudRate((uint8_t)gFlashMirror.ComBaud);
	}
    sendCRLF();

    sendAllB('M');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(gcMidiBaudStr);
	sendAllB(':');
	sendAllB(' ');
    if (gRuntimeSetup.MidiBaud == 0xFFFF) msgbyte = 31;
    else msgbyte = (uint8_t)gRuntimeSetup.MidiBaud;
	sendBaudRate(msgbyte);
    sendCRLF();

    sendAllB('C');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(gcChannelStr);
	sendAllB(':');
	sendAllB(' ');
    if (gRuntimeSetup.Channel == 0xFFFF) msgbyte = 1;
	else msgbyte = ((uint8_t)gRuntimeSetup.Channel) + 1;
//    if (msgbyte > 9) sendAllB((msgbyte/10)  + 0x30);
//    sendAllB((msgbyte %10) + 0x30);
	sendAllS12(msgbyte);
    sendCRLF();

    sendAllB('H');
    sendAllB(':');
    sendAllB(0x20);
    sendAllString(gcHysteresisStr);
	sendAllB(':');
	sendAllB(' ');
    if (gRuntimeSetup.Hysteresis == 0xFFFF) msgbyte = 2;
    else msgbyte = (uint8_t)gRuntimeSetup.Hysteresis;
    sendAllB(msgbyte + 0x30);
    sendCRLF();

    sendAllString(gcNumBStr);
	if (gNumBase == 0) sendAllString(gcDecimalStr);
	else sendAllString(gcHexStr);
    sendCRLF();

	
//activated input change messages (kept stored even if msgs are switched off)
	sendAllString(inputMsgStr);
    sendCRLF();
    for (k=1 ; k <= IO_NUM; k++)  {
        msgPtr_S= &(gRuntimeSetup.PidH) + (k<<1);
        msgPtr_D= msgPtr_S + 1;
		if (*msgPtr_S <= 0x000F)  {
			statusType = (uint8_t)*msgPtr_S;
			fullStatus = (uint8_t)(((statusType << 4) & 0x00F0) |
													gRuntimeSetup.Channel);
		}
		else  {
			statusType = (uint8_t)(*msgPtr_S >> 4) & 0x0F;
			fullStatus = (uint8_t)(*msgPtr_S & 0x00FF);
		}
        sendAllString(gcInputStr[k-1]);
        if ((*msgPtr_S & *msgPtr_D) == 0xFFFF)  sendAllString(gcNoMsgStr);
        else {
            sendAllString(msgStr);
            sendAllX8(fullStatus);
            if (fullStatus <= 0xF3)  {			//not a single byte message
                if ((fullStatus == 0xF2) || (statusType < 0x0C)) {
													//is a three byte message
					if (gNumBase)  {
						sendAllX8((uint8_t)(*msgPtr_D & 0x007F)); 
						sendAllX8((uint8_t)((*msgPtr_D >> 8) & 0x007F));
					}
					else  {
						sendAllS12(*msgPtr_D & 0x007F);
						sendAllS12((*msgPtr_D >> 8) & 0x007F);
					}
					if (*msgPtr_D < 0x8000) sendAllString(dig2Str);
					else sendAllString(ana7Str);
				}
				else if (statusType == 0x0E) {
													//is a three byte message
					if (gNumBase)  {
						sendAllX8((uint8_t)(*msgPtr_D & 0x00FF)); 
						sendAllX8((uint8_t)((*msgPtr_D >> 8) & 0x00FF));
					}
					else  {
						sendAllS12(*msgPtr_D & 0x00FF);
						sendAllS12((*msgPtr_D >> 8) & 0x00FF);
					}
					if ((*msgPtr_D & 0xFF00)==0x8100) sendAllString(ana12Str);
					else if ((*msgPtr_D&0xFF00)==0x8000)sendAllString(ana7Str);
					else sendAllString(dig2Str);
                }
				else  {								//is a 2 byte message
					if (gNumBase) sendAllX8((uint8_t)((*msgPtr_D>>8)&0x00FF));
					else  sendAllS12((*msgPtr_D >> 8) & 0x00FF);
					sendAllB(' ');
					sendAllB(' ');
					sendAllB(' ');
					if (*msgPtr_D < 0x8000) sendAllString(digStr);
					else sendAllString(ana7Str);
				}
            }
			else  {
				for (n=0;n<7;n++)  sendAllB(' ');
				sendAllString(singleStr);
			}
        }
        if (k != (IO_NUM)) sendCRLF();
    }
/*
uint16_t *pointer;
	sendCRLF();
	for (k= 1; k <= IO_NUM;k++) {
		pointer = &(gRuntimeSetup.PidH) +(k<<1);
		sendAllX16(*pointer++);
		sendAllX16(*pointer);
		sendCRLF();
	}
*/
}


void sendBaudRate(const uint8_t shortValue) // *******************************
// ***************************************************************************
{
	sendAllB((shortValue/10)  + 0x30);
    sendAllB((shortValue %10) + 0x30);
    switch (shortValue) {
        case 11:
            sendAllB('5');
            sendAllB('2');
            sendAllB('0');
            sendAllB('0');
            break;
        case 19:
            sendAllB('2');
            sendAllB('0');
            sendAllB('0');
            sendAllB(0x20);
            break;
        case 31:
            sendAllB('M');
            sendAllB('I');
            sendAllB('D');
            sendAllB('I');
            break;
        case 38:
            sendAllB('4');
            sendAllB('0');
            sendAllB('0');
            sendAllB(0x20);
            break;
        case 57:
            sendAllB('6');
            sendAllB('0');
            sendAllB('0');
            sendAllB(0x20);
            break;
        case 96:
            sendAllB('0');
            sendAllB('0');
            sendAllB(0x20);
            sendAllB(0x20);
            break;
        default:
            sendAllB('?');
            sendAllB('?');
            sendAllB('?');
            sendAllB(0x20);
    }
}

// end of file
